/**
 * @author Marcel Smit <msmit@remedy.nl>
 *
 * $Id: DDS_Event_Connector_T.h 87719 2009-11-24 09:35:53Z johnnyw $
 *
 * Wrapper facade for NDDS.
 */
#ifndef CONNECTOR_T_H_
#define CONNECTOR_T_H_

#include "dds4ccm/impl/ndds/DDS_TopicBase_Connector_T.h"

template <typename DDS_TYPE, typename CCM_TYPE>
class DDS_Event_Connector_T
  : public virtual DDS_TopicBase_Connector_T<DDS_TYPE, CCM_TYPE>
{
public:
  DDS_Event_Connector_T (void);
  virtual ~DDS_Event_Connector_T (void);

  /**
   * @name DDS_Write
   * DDS_Write operations
   */
  //@{
  virtual typename CCM_TYPE::writer_type::_ptr_type get_supplier_data (void);

  virtual ::DDS::CCM_DataWriter_ptr get_supplier_dds_entity (void);
  //@}

  /**
   * @name DDS_Get
   * DDS_Get operations
   */
  //@{
  virtual typename CCM_TYPE::reader_type::_ptr_type get_pull_consumer_data (void);

  virtual typename CCM_TYPE::getter_type::_ptr_type get_pull_consumer_fresh_data (void);

  virtual ::DDS::CCM_DataReader_ptr get_pull_consumer_dds_entity (void);
  //@}

  /**
   * @name DDS_Listen
   * DDS_Listen operations
   */
  //@{
  virtual typename CCM_TYPE::reader_type::_ptr_type get_push_consumer_data (void);

  virtual ::CCM_DDS::CCM_DataListenerControl_ptr get_push_consumer_data_control (void);

  virtual ::DDS::CCM_DataReader_ptr get_push_consumer_dds_entity (void);
  //@}

  virtual void configuration_complete (void);

  virtual void ccm_activate (void);
  virtual void ccm_passivate (void);
  virtual void ccm_remove (void);

private:
  // @from DDS_Write for
  void configure_port_dds_write (void);
  ::DDS::Publisher_var supplier_publisher_;
  ::DDS::CCM_DataWriter_var supplier_writer_;
  ::DDS::DataWriterListener_var supplier_listener_;

  // @from DDS_Listen
  void configure_port_dds_listen (void);
  ACE_Atomic_Op <TAO_SYNCH_MUTEX, ::CCM_DDS::ListenerMode> listen_datalistener_mode_;
  ACE_Atomic_Op <TAO_SYNCH_MUTEX, ::CCM_DDS::DataNumber_t> listen_datalistener_max_delivered_data_;
  ::DDS::Subscriber_var listen_subscriber_;
  ::DDS::DataReader_var push_consumer_data_;
  ::DDS::DataReaderListener_var __listen_datareaderlistener;

  // @from DDS_Getter
  ::DDS::DataReader_var pull_consumer_fresh_data_;
};

#include "dds4ccm/impl/ndds/DDS_Event_Connector_T.cpp"

#endif /* CONNECTOR_T_H_ */
